/*
 ThermostatQA - test framework for Thermostat Monitoring Tool

 Copyright 2015 Red Hat, Inc.

 This file is part of ThermostatQA

 ThermostatQA is distributed under the GNU General Public License,
 version 2 or any later version (with a special exception described
 below, commonly known as the "Classpath Exception").

 A copy of GNU General Public License (GPL) is included in this
 distribution, in the file COPYING.

 Linking ThermostatQA code with other modules is making a combined work
 based on ThermostatQA.  Thus, the terms and conditions of the GPL
 cover the whole combination.

 As a special exception, the copyright holders of ThermostatQA give you
 permission to link this code with independent modules to produce an
 executable, regardless of the license terms of these independent
 modules, and to copy and distribute the resulting executable under
 terms of your choice, provided that you also meet, for each linked
 independent module, the terms and conditions of the license of that
 module.  An independent module is a module which is not derived from
 or based on ThermostatQA code.  If you modify ThermostatQA, you may
 extend this exception to your version of the software, but you are
 not obligated to do so.  If you do not wish to do so, delete this
 exception statement from your version.
 */
package org.thermostat.qa2.framework.utils;

import java.lang.annotation.Annotation;
import java.lang.reflect.AccessibleObject;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import org.thermostat.qa2.framework.annotations.InheritAnnotations;

/**
 *
 * @author Zdeněk Žamberský
 */
public class AnnotationUtilities {

    public static Annotation[] getAnnotations(Object o) {
        if (o instanceof Class) {
            Class oc = (Class) o;
            return oc.getAnnotations();
        } else if (o instanceof AccessibleObject) {
            AccessibleObject ao = (AccessibleObject) o;
            return ao.getAnnotations();
        } else {
            throw new IllegalArgumentException("Annotation cannot be read from instance of class: " + o.getClass());
        }
    }

    public static <T extends AccessibleObject> List getAnnotatedObjects(List<T> objects, Class annotationType) {
        ArrayList<T> annotated = new ArrayList();
        for (T o : objects) {
            if (o.isAnnotationPresent(annotationType)) {
                annotated.add(o);
            }
        }
        return annotated;
    }

    public static <A extends Annotation> A findAnnotation(Object o, Class<A> c) {
        if (o instanceof Class) {
            Class oc = (Class) o;
            return (A) oc.getAnnotation(c);
        } else if (o instanceof AccessibleObject) {
            AccessibleObject ao = (AccessibleObject) o;
            return ao.getAnnotation(c);
        } else {
            throw new IllegalArgumentException("Annotation cannot be read from instance of class: " + o.getClass());
        }
    }

    // also recursivly search in annotations annotating annotatations
    public static <A extends Annotation> A findAnnotationWithInheritance(Object o, Class<A> c) {
        A annotation = findAnnotation(o, c);
        if (annotation != null) {
            return annotation;
        }
        Annotation[] annotations = getAnnotations(o);
        for (Annotation a : annotations) {
            Class type = a.annotationType();
            if (findAnnotation(type, InheritAnnotations.class) != null) {
                annotation = findAnnotationWithInheritance(type, c);
                if (annotation != null) {
                    return annotation;
                }
            }
        }
        return null;
    }

    public static <A extends Annotation> A findAnnotationWithClassInheritance(AccessibleObject member, Class owningClass, Class<A> annotationClass) {
        A annotation;
        annotation = AnnotationUtilities.findAnnotation(member, annotationClass);
        if (annotation != null) {
            return annotation;
        }
        annotation = AnnotationUtilities.findAnnotation(owningClass, annotationClass);
        if (annotation != null) {
            return annotation;
        }
        annotation = AnnotationUtilities.findAnnotationWithInheritance(member, annotationClass);
        if (annotation != null) {
            return annotation;
        }
        annotation = AnnotationUtilities.findAnnotationWithInheritance(owningClass, annotationClass);
        return annotation;
    }

    public static <A extends Annotation> A findAnnotationWithClassInheritance(Method method, Class<A> annotationClass) {
        return findAnnotationWithClassInheritance(method, method.getDeclaringClass(), annotationClass);
    }

}
